package controllers

import (
	"net/http"
	"spaceofcheng/services/service"
)

// Authenticate 权限验证
// 其中 token 的处理方式是
// 1、获取 request 中 token
// 2、放入 Context 中
// 3、校验 token
// 4、结束后，设置过期并从 Context 中删除
// 5、生成新的 token， 并放入 Context 中
func (c *BaseController) authenticate() {
	serv := service.NewSysServ(c.Context)

	// 鉴权 - 并初始化上下文
	res := serv.AuthAndInitCtx(c.Ctx)

	if res != nil {
		code := http.StatusOK
		if res["code"] != nil {
			code = res["code"].(int)
		}

		if code != http.StatusOK {
			err := res["error"].(error)
			data := map[string]interface{}{}

			if res["message"] != nil {
				data = res["message"].(map[string]interface{})
			}

			// 设置旧 token 过期
			c.SetTokenExipre()

			c.ResponseData(data, err, code)
		}
	}

	// 设置旧 token 过期
	c.SetTokenExipre()

	// 生成新 token
	c.CreateNewToken()
}

// SetTokenExipre 设置 token 过期
func (c *BaseController) SetTokenExipre() {
	token := c.Context.Get("token")
	if token != nil {
		serv := service.NewSysServ(c.Context)
		serv.UpdateTokenExpire(token.(string))
	}

	c.Context.Set("token", "")
}

// CreateNewToken 新 token
func (c *BaseController) CreateNewToken() {
	serv := service.NewSysServ(c.Context)
	c.Context.Set("token", serv.NewToken())
}
