package utils

import (
	"io"
	"reflect"

	"github.com/tealeg/xlsx"
)

// TinyXLSXEngine 简版 xlsx 处理器
type TinyXLSXEngine struct {
	xlsxFile *xlsx.File
	toSlice  func(interface{}) []interface{}
}

// NewTinyXLSXEngine init
func NewTinyXLSXEngine() *TinyXLSXEngine {
	return &TinyXLSXEngine{}
}

// SetTransFunc 设置转换函数
func (t *TinyXLSXEngine) SetTransFunc(f func(interface{}) []interface{}) *TinyXLSXEngine {
	t.toSlice = f
	return t
}

// SetData 设置数据
func (t *TinyXLSXEngine) SetData(data interface{}) *TinyXLSXEngine {
	file := xlsx.NewFile()
	t.xlsxFile = file

	sheet, err := file.AddSheet("sheet1")
	if err != nil {
		LogError("创建 xlsx sheet 失败: " + err.Error())
		return t
	}

	if t.toSlice == nil {
		t.toSlice = func(dt interface{}) []interface{} {
			return nil
		}
	}

	v := reflect.ValueOf(data)
	if v.Kind() == reflect.Ptr {
		v.Elem()
	}

	if v.Kind() == reflect.Slice && v.IsValid() {
		l := v.Len()
		for i := 0; i < l; i++ {
			rowData := v.Index(i).Interface()
			cols := t.toSlice(rowData)

			row := sheet.AddRow()
			for _, col := range cols {
				row.AddCell().SetValue(col)
			}
		}
	}

	return t
}

// Write 写数据
func (t *TinyXLSXEngine) Write(w io.Writer) error {
	return t.xlsxFile.Write(w)
}
